# Copyright (c) 2010 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abrek.testdef

RUNSTEPS = ["glmemperf -e shmimage"]
PATTERN = "^(?P<test_case_id>\w+):\W+(?P<measurement>\d+) fps"

inst = abrek.testdef.AbrekTestInstaller(deps=["glmemperf"])
run = abrek.testdef.AbrekTestRunner(RUNSTEPS)
parse = abrek.testdef.AbrekTestParser(PATTERN,
                                      appendall={'units':'fps',
                                                 'result':'pass'})

testobj = abrek.testdef.AbrekTest(testname="glmemperf", installer=inst,
                                  runner=run, parser=parse)



# Copyright (c) 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abrek.testdef

INSTALLSTEPS = ['bzr branch lp:~linaro-foundations/lava-test/bootchartscript']
DEPS = ['bootchart', 'pybootchartgui', 'bzr']
RUNSTEPS = ['./bootchartscript/bootchartscript.sh']
PATTERN = "^(?P<test_case_id>\w+):\W+(?P<measurement>\d+\.\d+)"

bootchartinst = abrek.testdef.AbrekTestInstaller(INSTALLSTEPS, deps=DEPS)
bootchartrun = abrek.testdef.AbrekTestRunner(RUNSTEPS)
bootchartparser = abrek.testdef.AbrekTestParser(PATTERN,
                   appendall={'units':'sec', 'result':'pass'})
testobj = abrek.testdef.AbrekTest(testname="bootchart", installer=bootchartinst,
                                  runner=bootchartrun, parser=bootchartparser)



# Copyright (c) 2010 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
   This script automates the automate installation, execution, and
   results parsing for the GMPbench test suite.
   The GMPbench test Suite is an open source test suite with the goal of
   comparing different processors performance against eachother.

"""

import abrek.testdef

VERSION='0.2'
URL="ftp://ftp.gmplib.org/pub/misc/gmpbench-%s.tar.bz2" %(VERSION)
URL_gexpr="http://www.gmplib.org/gexpr.c"
DEPS = ['gcc', 'libgmp3-dev', 'wget', 'bzip2']

INSTALLSTEPS = ['tar -xjf  gmpbench-0.2.tar.bz2',
                'wget -c %s' %(URL_gexpr),
                'mv gexpr.c gmpbench-0.2',
                'cd gmpbench-0.2 && gcc -o gexpr gexpr.c  -static -lm']
RUNSTEPS = ['cd  gmpbench-0.2 && PATH=$PATH:. ./runbench ']
PATTERN = "\s*(?P<test_case_id>GMPbench\.*\w*\.*\w*):?\s*"\
          "(?P<measurement>\d+.\d+)"

gmpbenchinst = abrek.testdef.AbrekTestInstaller(INSTALLSTEPS, deps=DEPS,
                                                url=URL)
gmpbenchrun = abrek.testdef.AbrekTestRunner(RUNSTEPS)
gmpbenchparser = abrek.testdef.AbrekTestParser(PATTERN,
    appendall={'units':'operations/s', 'result':'pass'})
testobj = abrek.testdef.AbrekTest(testname="gmpbench", installer=gmpbenchinst,
    runner=gmpbenchrun, parser=gmpbenchparser)



# Copyright (c) 2010 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abrek.testdef


INSTALLSTEPS = ['git clone git://git.linaro.org/people/torez/pm-qa.git',
                'cd pm-qa && make clean && make all']
RUNSTEPS = ['cd pm-qa && awk -f testcases.awk  run_template']
DEPS = ['git-core', 'make', 'alsa-utils', 'pulseaudio-utils', 'lame', 'festival', 'wget']

pwrmgmtinst = abrek.testdef.AbrekTestInstaller(INSTALLSTEPS, deps=DEPS)
pwrmgmtrun = abrek.testdef.AbrekTestRunner(RUNSTEPS)

# test case name is between "pm-qa-"  and  ":"  and results and/or
# measurements are rest of the line
PATTERN = "^pm-qa-(?P<test_case_id>\w+):\s+(?P<message>.*)"


pwrmgmtparser = abrek.testdef.AbrekTestParser(PATTERN,
    appendall={'result':'pass'})

testobj = abrek.testdef.AbrekTest(testname="pwrmgmt", installer=pwrmgmtinst,
                                  runner=pwrmgmtrun, parser=pwrmgmtparser)



"""
Test providers.

Allow listing and loading of tests in a generic way.
"""

from pkg_resources import working_set

from abrek.api import ITestProvider
from abrek.cache import AbrekCache
from abrek.config import get_config
from abrek.testdef import AbrekDeclarativeTest


class BuiltInProvider(ITestProvider):
    """
    Test provider that provides tests shipped in the Abrek source tree
    """

    _builtin_tests = [
        'bootchart',
        'firefox',
        'glmemperf',
        'gmpbench',
        'gtkperf',
        'ltp',
        'peacekeeper'
        'posixtestsuite',
        'pwrmgmt',
        'pybench',
        'smem',
        'stream',
        'tiobench',
        'x11perf',
        'xrestop',
    ]

    def __init__(self, config):
        pass

    @property
    def description(self):
        return "Tests built directly into Abrek:"

    def __iter__(self):
        return iter(self._builtin_tests)

    def __getitem__(self, test_name):
        try:
            module = __import__("abrek.test_definitions.%s" % test_name, fromlist=[''])
        except ImportError:
            raise KeyError(test_name)
        else:
            return module.testobj


class PkgResourcesProvider(ITestProvider):
    """
    Test provider that provides tests declared in pkg_resources working_set

    By default it looks at the 'abrek.test_definitions' name space but it can
    be changed with custom 'namespace' configuration entry.
    """

    def __init__(self, config):
        self._config = config

    @property
    def namespace(self):
        return self._config.get("namespace", "abrek.test_definitions")

    @property
    def description(self):
        return "Tests provided by installed python packages:"

    def __iter__(self):
        for entry_point in working_set.iter_entry_points(self.namespace):
            yield entry_point.name

    def __getitem__(self, test_name):
        for entry_point in working_set.iter_entry_points(self.namespace, test_name):
            return entry_point.load().testobj
        raise KeyError(test_name)


class RegistryProvider(ITestProvider):
    """
    Test provider that provides declarative tests listed in the test registry.
    """
    def __init__(self, config):
        self._config = config
        self._cache = None 

    @property
    def entries(self):
        """
        List of URLs to AbrekDeclarativeTest description files
        """
        return self._config.get("entries", [])

    @property
    def description(self):
        return "Tests provided by Abrek registry:"

    @classmethod
    def register_remote_test(self, test_url):
        config = get_config() # This is a different config object from self._config
        provider_config = config.get_provider_config("abrek.providers:RegistryProvider")
        if "entries" not in provider_config:
            provider_config["entries"] = []
        if test_url not in provider_config["entries"]:
            provider_config["entries"].append(test_url)
            config._save_registry()
        else:
            raise ValueError("This test is already registered")

    def _load_remote_test(self, test_url):
        """
        Load AbrekDeclarativeTest from a (possibly cached copy of) test_url
        """
        cache = AbrekCache.get_instance()
        with cache.open_cached_url(test_url) as stream:
            return AbrekDeclarativeTest.load_from_stream(stream)

    def _fill_cache(self):
        """
        Fill the cache of all remote tests
        """
        if self._cache is not None:
            return
        self._cache = {}
        for test_url in self.entries:
            test = self._load_remote_test(test_url)
            if test.testname in self._cache:
                raise ValueError("Duplicate test %s declared" % test.testname)
            self._cache[test.testname] = test

    def __iter__(self):
        self._fill_cache()
        for test_name in self._cache.iterkeys():
            yield test_name

    def __getitem__(self, test_name):
        self._fill_cache()
        return self._cache[test_name]



"""
Cache module for Abrek
"""
import contextlib
import hashlib
import os
import urllib2


class AbrekCache(object):
    """
    Cache class for Abrek
    """

    _instance = None

    def __init__(self):
        home = os.environ.get('HOME', '/')
        basecache = os.environ.get('XDG_CACHE_HOME',
                     os.path.join(home, '.cache'))
        self.cache_dir = os.path.join(basecache, 'abrek')

    @classmethod
    def get_instance(cls):
        if cls._instance is None:
            cls._instance = cls()
        return cls._instance

    def open_cached(self, key, mode="r"):
        """
        Acts like open() but the pathname is relative to the
        abrek-specific cache directory.
        """
        if "w" in mode and not os.path.exists(self.cache_dir):
            os.makedirs(self.cache_dir)
        if os.path.isabs(key):
            raise ValueError("key cannot be an absolute path")
        try:
            stream = open(os.path.join(self.cache_dir, key), mode)
            yield stream
        finally:
            stream.close()

    def _key_for_url(self, url):
        return hashlib.sha1(url).hexdigest()

    def _refresh_url_cache(self, key, url):
        with contextlib.nested(
            contextlib.closing(urllib2.urlopen(url)),
            self.open_cached(key, "wb")) as (in_stream, out_stream):
            out_stream.write(in_stream.read())

    @contextlib.contextmanager
    def open_cached_url(self, url):
        """
        Like urlopen.open() but the content may be cached.
        """
        # Do not cache local files, this is not what users would expect

        # workaround - not using cache at all.
        # TODO: fix this and use the cache
        # if url.startswith("file://"):
        if True:
            stream = urllib2.urlopen(url)
        else:
            key = self._key_for_url(url)
            try:
                stream = self.open_cached(key, "rb")
            except IOError as exc:
                self._refresh_url_cache(key, url)
                stream = self.open_cached(key, "rb")
        yield stream
        stream.close()



# Copyright (c) 2010 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import hashlib
import json
import os
import re
import shutil
import decimal
import sys
import time
from commands import getstatusoutput
from datetime import datetime
from uuid import uuid4

from abrek import swprofile, hwprofile
from abrek.api import ITest
from abrek.bundle import DocumentIO
from abrek.config import get_config
from abrek.utils import geturl, run_and_log, write_file


class AbrekTest(ITest):
    """Base class for defining tests.

    This can be used by test definition files to create an object that
    contains the building blocks for installing tests, running them,
    and parsing the results.

    testname - name of the test or test suite
    version - version of the test or test suite
    installer - AbrekInstaller instance to use
    runner - AbrekRunner instance to use
    parser - AbrekParser instance to use
    """
    def __init__(self, testname, version="", installer=None, runner=None,
                 parser=None):
        self.testname = testname
        self.version = version
        self.installer = installer
        self.runner = runner
        self.parser = parser
        self.origdir = os.path.abspath(os.curdir)

    def install(self):
        """Install the test suite.

        This creates an install directory under the user's XDG_DATA_HOME
        directory to mark that the test is installed.  The installer's
        install() method is then called from this directory to complete any
        test specific install that may be needed.
        """
        if not self.installer:
            raise RuntimeError("no installer defined for '%s'" %
                                self.testname)
        config = get_config()
        installdir = os.path.join(config.installdir, self.testname)
        if os.path.exists(installdir):
            raise RuntimeError("%s is already installed" % self.testname)
        os.makedirs(installdir)
        os.chdir(installdir)
        try:
            self.installer.install()
        except Exception as strerror:
            self.uninstall()
            raise
        finally:
            os.chdir(self.origdir)

    def uninstall(self):
        """Uninstall the test suite.

        Uninstalling just recursively removes the test specific directory
        under the user's XDG_DATA_HOME directory.  This will both mark
        the test as removed, and clean up any files that were downloaded
        or installed under that directory.  Dependencies are intentionally
        not removed by this.
        """
        os.chdir(self.origdir)
        config = get_config()
        path = os.path.join(config.installdir, self.testname)
        if os.path.exists(path):
            shutil.rmtree(path)

    def _savetestdata(self, analyzer_assigned_uuid):
        TIMEFORMAT = '%Y-%m-%dT%H:%M:%SZ'
        bundle = {
            'format': 'Dashboard Bundle Format 1.2',
            'test_runs': [
                {
                    'test_id': self.testname,
                    'analyzer_assigned_date': self.runner.starttime.strftime(TIMEFORMAT),
                    'analyzer_assigned_uuid': analyzer_assigned_uuid,
                    'time_check_performed': False,
                    'hardware_context': hwprofile.get_hardware_context(),
                    'software_context': swprofile.get_software_context(),
                    'test_results': []
                }
            ]
        }
        filename = os.path.join(self.resultsdir, 'testdata.json')
        write_file(DocumentIO.dumps(bundle), filename)

    def run(self, quiet=False):
        if not self.runner:
            raise RuntimeError("no test runner defined for '%s'" %
                                self.testname)
        config = get_config()
        uuid = str(uuid4())
        installdir = os.path.join(config.installdir, self.testname)
        resultname = (self.testname +
                     str(time.mktime(datetime.utcnow().timetuple())))
        self.resultsdir = os.path.join(config.resultsdir, resultname)
        os.makedirs(self.resultsdir)
        try:
            os.chdir(installdir)
            self.runner.run(self.resultsdir, quiet=quiet)
            self._savetestdata(uuid)
        finally:
            os.chdir(self.origdir)
        result_id = os.path.basename(self.resultsdir)
        print("ABREK TEST RUN COMPLETE: Result id is '%s'" % result_id)
        return result_id

    def parse(self, resultname):
        if not self.parser:
            raise RuntimeError("no test parser defined for '%s'" %
                                self.testname)
        config = get_config()
        resultsdir = os.path.join(config.resultsdir, resultname)
        os.chdir(resultsdir)
        self.parser.parse()
        os.chdir(self.origdir)


class AbrekDeclarativeTest(AbrekTest):
    """
    Declarative test is like AbrekTest but cannot contain any python code and
    is completely encapsulated in .json file. 
    """

    @classmethod
    def load_from_stream(cls, stream):
        return cls(json.load(stream))

    def save_to_stream(self, stream):
        json.dumps(self.about, stream, indent="2")

    def __init__(self, about):
        self.about = about
        super(AbrekDeclarativeTest, self).__init__(self.about.get('test_id'))
        self.installer = AbrekTestInstaller(**self.about.get('install', {}))
        self.runner = AbrekTestRunner(**self.about.get('run', {}))
        if self.about.get('parse', {}).get('native', False) is True:
            self.parser = AbrekNativeTestParser(self)
        else:
            self.parser = AbrekForeignTestParser(**self.about.get('parse', {}))


class AbrekTestInstaller(object):
    """Base class for defining an installer object.

    This class can be used as-is for simple installers, or extended for more
    advanced funcionality.

    steps - list of steps to be executed in a shell
    deps - list of dependencies to apt-get install before running the steps
    url - location from which the test suite should be downloaded
    md5 - md5sum to check the integrety of the download
    """
    def __init__(self, steps=[], deps=[], url="", md5="", **kwargs):
        self.steps = steps
        self.deps = deps
        self.url = url
        self.md5 = md5

    def _installdeps(self):
        if not self.deps:
            return 0
        cmd = "sudo apt-get install -y %s" % " ".join(self.deps)
        rc, output = getstatusoutput(cmd)
        if rc:
            raise RuntimeError("Dependency installation failed. %d : %s" %(rc,output))

    def _download(self):
        """Download the file specified by the url and check the md5.

        Returns the path and filename if successful, otherwise return None
        """
        if not self.url:
            return 0
        filename = geturl(self.url)
        #If the file does not exist, then the download was not successful
        if not os.path.exists(filename):
            return None
        if self.md5:
            checkmd5 = hashlib.md5()
            with open(filename, 'rb') as fd:
                data = fd.read(0x10000)
                while data:
                    checkmd5.update(data)
                    data = fd.read(0x10000)
            if checkmd5.hexdigest() != self.md5:
                raise RuntimeError("Unexpected md5sum downloading %s" %
                                    filename)
                return None
        return filename

    def _runsteps(self):
        for cmd in self.steps:
            rc, output = getstatusoutput(cmd)
            if rc:
                raise RuntimeError("Run step '%s' failed. %d : %s" %(cmd,rc,output))


    def install(self):
        self._installdeps()
        self._download()
        self._runsteps()


class AbrekTestRunner(object):
    """Base class for defining an test runner object.

    This class can be used as-is for simple execution with the expectation
    that the run() method will be called from the directory where the test
    was installed.  Steps, if used, should handle changing directories from
    there to the directory where the test was extracted if necessary.
    This class can also be extended for more advanced funcionality.

    steps - list of steps to be executed in a shell
    """
    def __init__(self, steps=[]):
        self.steps = steps
        self.testoutput = []

    def _runsteps(self, resultsdir, quiet=False):
        outputlog = os.path.join(resultsdir, 'testoutput.log')
        with open(outputlog, 'a') as fd:
            for cmd in self.steps:
                run_and_log(cmd, fd, quiet)

    def run(self, resultsdir, quiet=False):
        self.starttime = datetime.utcnow()
        self._runsteps(resultsdir, quiet=quiet)
        self.endtime = datetime.utcnow()


class AbrekForeignTestParser(object):
    """Base class for defining a test parser

    This class can be used as-is for simple results parsers, but will
    likely need to be extended slightly for many.  If used as it is,
    the parse() method should be called while already in the results
    directory and assumes that a file for test output will exist called
    testoutput.log.

    pattern - regexp pattern to identify important elements of test output
        For example: If your testoutput had lines that look like:
            "test01:  PASS", then you could use a pattern like this:
            "^(?P<testid>\w+):\W+(?P<result>\w+)"
            This would result in identifying "test01" as testid and
            "PASS" as result.  Once parse() has been called,
            self.results.test_results[] contains a list of dicts of all the
            key,value pairs found for each test result
    fixupdict - dict of strings to convert test results to standard strings
        For example: if you want to standardize on having pass/fail results
            in lower case, but your test outputs them in upper case, you could
            use a fixupdict of something like: {'PASS':'pass','FAIL':'fail'}
    appendall - Append a dict to the test_results entry for each result.
        For example: if you would like to add units="MB/s" to each result:
            appendall={'units':'MB/s'}
    """
    def __init__(self, pattern=None, fixupdict=None, appendall={}):
        self.pattern = pattern
        self.fixupdict = fixupdict
        self.results = {'test_results':[]}
        self.appendall = appendall

    def _find_testid(self, id):
        for x in self.results['test_results']:
            if x['testid'] == id:
                return self.results['test_results'].index(x)

    def parse(self):
        """Parse test output to gather results

        Use the pattern specified when the class was instantiated to look
        through the results line-by-line and find lines that match it.
        Results are then stored in self.results.  If a fixupdict was supplied
        it is used to convert test result strings to a standard format.
        """
        filename = "testoutput.log"
        try:
            pat = re.compile(self.pattern)
        except Exception as strerror:
            raise RuntimeError(
                "AbrekTestParser - Invalid regular expression '%s' - %s" % (
                    self.pattern,strerror))

        with open(filename, 'r') as stream:
            for lineno, line in enumerate(stream, 1):
                match = pat.search(line)
                if not match:
                    continue
                data = match.groupdict()
                if 'measurement' in data:
                    data['measurement'] = decimal.Decimal(data['measurement'])
                data["log_filename"] = filename
                data["log_lineno"] = lineno
                self.results['test_results'].append(data)
        if self.fixupdict:
            self.fixresults(self.fixupdict)
        if self.appendall:
            self.appendtoall(self.appendall)
        self.fixmeasurements()
        self.fixids()

    def append(self, testid, entry):
        """Appends a dict to the test_results entry for a specified testid

        This lets you add a dict to the entry for a specific testid
        entry should be a dict, updates it in place
        """
        index = self._find_testid(testid)
        self.results['test_results'][index].update(entry)

    def appendtoall(self, entry):
        """Append entry to each item in the test_results.

        entry - dict of key,value pairs to add to each item in the
        test_results
        """
        for t in self.results['test_results']:
            t.update(entry)

    def fixresults(self, fixupdict):
        """Convert results to a known, standard format

        pass it a dict of keys/values to replace
        For instance:
            {"TPASS":"pass", "TFAIL":"fail"}
        This is really only used for qualitative tests
        """
        for t in self.results['test_results']:
            if t.has_key("result"):
                t['result'] = fixupdict[t['result']]

    def fixmeasurements(self):
        """Measurements are often read as strings, but need to be float
        """
        for id in self.results['test_results']:
            if id.has_key('measurement'):
                id['measurement'] = float(id['measurement'])

    def fixids(self):
        """
        Convert spaces to _ in test_case_id and remove illegal characters
        """
        badchars = "[^a-zA-Z0-9\._-]"
        for id in self.results['test_results']:
            if id.has_key('test_case_id'):
                id['test_case_id'] = id['test_case_id'].replace(" ", "_")
                id['test_case_id'] = re.sub(badchars, "", id['test_case_id'])


AbrekTestParser = AbrekForeignTestParser


class AbrekNativeTestParser(object):

    def __init__(self, test_def):
        self.test_def = test_def

    def parse(self):
        raise NotImplementedError(self.parse)


class TestLoader(object):
    """
    Test loader.

    Encapsulates Abrek's knowledge of available tests.

    Test can be loaded by name with TetsLoader.get_test_by_name. Test can also
    be listed by TestLoader.get_providers() and then iterating over tests
    returned by each provider.
    """

    def __init__(self):
        self._config = get_config()

    def get_providers(self):
        """
        Return a generator of available providers
        """
        import pkg_resources
        for provider_info in self._config.registry.get("providers", []):
            entry_point_name = provider_info.get("entry_point")
            module_name, attrs = entry_point_name.split(':', 1)
            attrs = attrs.split('.')
            try:
                entry_point = pkg_resources.EntryPoint(entry_point_name, module_name, attrs,
                                                   dist=pkg_resources.get_distribution("lava-test"))
                provider_cls = entry_point.load()
                provider = provider_cls(provider_info.get("config", {}))
                yield provider
            except pkg_resources.DistributionNotFound as ex:
                raise RuntimeError("lava-test is not properly set up. Please read the REAMME file")

    def get_test_by_name(self, test_name):
        """
        Lookup a test with the specified name
        """
        for provider in self.get_providers():
            try:
                return provider[test_name]
            except KeyError:
                pass
        raise ValueError("No such test %r" % test_name)


testloader = TestLoader().get_test_by_name



# Copyright (c) 2010 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import shutil
import sys
from optparse import make_option

from abrek.command import AbrekCmd, AbrekCmdWithSubcommands
from abrek.config import get_config
from abrek.utils import read_file


class cmd_results(AbrekCmdWithSubcommands):
    """
    Operate on results of previous test runs stored locally
    """

    class cmd_list(AbrekCmd):
        """
        List results of previous runs
        """
        def run(self):
            config = get_config()
            print "Saved results:"
            try:
                for dir in os.listdir(config.resultsdir):
                    print dir
            except OSError:
                print "No results found"


    class cmd_show(AbrekCmd):
        """
        Display the output from a previous test run
        """
        arglist = ['*result']
        def run(self):
            if len(self.args) != 1:
                print "please specify the name of the result dir"
                sys.exit(1)
            config = get_config()
            resultsdir = os.path.join(config.resultsdir, self.args[0])
            testoutput = os.path.join(resultsdir, "testoutput.log")
            if not os.path.exists(testoutput):
                print "No result found for '%s'" % self.args[0]
                sys.exit(1)
            try:
                print(read_file(testoutput))
            except IOError:
                pass


    class cmd_remove(AbrekCmd):
        """
        Remove the results of a previous test run
        """
        arglist = ['*result']
        options = [make_option('-f', '--force', action='store_true',
                               dest='force')]
        def run(self):
            if len(self.args) != 1:
                print "please specify the name of the result dir"
                sys.exit(1)
            config = get_config()
            resultsdir = os.path.join(config.resultsdir, self.args[0])
            if not os.path.exists(resultsdir):
                print "No result found for '%s'" % self.args[0]
                sys.exit(1)
            if not self.opts.force:
                print "Delete result '%s' for good? [Y/N]" % self.args[0],
                response = raw_input()
                if response[0].upper() != 'Y':
                    sys.exit(0)
            shutil.rmtree(resultsdir)


    class cmd_rename(AbrekCmd):
        """
        Rename the results from a previous test run
        """
        arglist = ['*source', '*destination']

        def run(self):
            if len(self.args) != 2:
                print "please specify the name of the result, and the new name"
                sys.exit(1)
            config = get_config()
            srcdir = os.path.join(config.resultsdir, self.args[0])
            destdir = os.path.join(config.resultsdir, self.args[1])
            if not os.path.exists(srcdir):
                print "Result directory not found"
                sys.exit(1)
            if os.path.exists(destdir):
                print "Destination result name already exists"
                sys.exit(1)
            shutil.move(srcdir, destdir)





# Copyright (c) 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
This module attempts to use the linaro-dashboard-bundle package, if possible.

Using that package adds proper support for loading and saving bundle
documents.  In particular it supports loosles decimals, better, more stable
load-modify-write cycles, data validation, transparent migration and many
other features.

It is not a hard dependency to make it possible to run abrek from a checkout
without having to install (too many) dependencies.
"""

try:
    from linaro_dashboard_bundle import DocumentIO
except ImportError:
    import json

    class DocumentIO(object):
        """ Bare replacement DocumentIO without any fancy features """

        @classmethod
        def dumps(cls, doc):
            return json.dumps(doc, indent=2)

        @classmethod
        def loads(cls, text):
            doc = json.loads(text)
            fmt = doc.get("format")
            return fmt, doc


# Copyright (c) 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abrek.testdef

RUNSTEPS = ['smem -w | tail -n 3']
PATTERN = "^(?P<test_case_id>(\w+\s)+)\s\s+(?P<measurement>\d+)"
DEPS = ['smem']

smeminst = abrek.testdef.AbrekTestInstaller(deps=DEPS)
smemrun = abrek.testdef.AbrekTestRunner(RUNSTEPS)
smemparser = abrek.testdef.AbrekTestParser(PATTERN,
               appendall={'units':'KB', 'result':'pass'})
testobj = abrek.testdef.AbrekTest(testname="smem", installer=smeminst,
                                  runner=smemrun, parser=smemparser)



# Copyright (c) 2010 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
   This script automates the automate installation, execution, and
   results parsing for the Peacekeeper browser benchmark.

   http://clients.futuremark.com/peacekeeper/index.action
"""

import abrek.testdef
import os

curdir = os.path.realpath(os.path.dirname(__file__))

INSTALLSTEPS = ['cp -rf %s/peacekeeper/* .'%curdir]
RUNSTEPS = ['python peacekeeper_runner.py firefox']
DEPS = ['python-ldtp','firefox']

my_installer = abrek.testdef.AbrekTestInstaller(INSTALLSTEPS, deps=DEPS)
my_runner = abrek.testdef.AbrekTestRunner(RUNSTEPS)

PATTERN = "^(?P<result>\w+): Score = (?P<measurement>\d+)"

my_parser = abrek.testdef.AbrekTestParser(PATTERN,
                                          appendall={'units':'point'})

testobj = abrek.testdef.AbrekTest(testname="peacekeeper", installer=my_installer,
                                  runner=my_runner, parser=my_parser)



# Copyright (c) 2010 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import shutil
import subprocess
import sys
import urllib2
import urlparse

_fake_files = None
_fake_paths = None
_fake_machine = None


class Tee(file):
    """ A file-like object that optionally mimics tee functionality.

    By default, output will go to both stdout and the file specified.
    Optionally, quiet=True can be used to mute the output to stdout.
    """
    def __init__(self, *args, **kwargs):
        try:
            self.quiet = kwargs.pop('quiet')
        except KeyError:
            self.quiet = False
        super(Tee, self).__init__(*args, **kwargs)

    def write(self, data):
        super(Tee, self).write(data)
        if self.quiet is False:
            sys.stdout.write(data)


def geturl(url, path=""):
    urlpath = urlparse.urlsplit(url).path
    filename = os.path.basename(urlpath)
    if path:
        filename = os.path.join(path, filename)
    fd = open(filename, "w")
    try:
        response = urllib2.urlopen(urllib2.quote(url, safe=":/"))
        fd = open(filename, 'wb')
        shutil.copyfileobj(response, fd, 0x10000)
        fd.close()
        response.close()
    except:
        raise RuntimeError("Could not retrieve %s" % url)
    return filename


def write_file(data, path):
    with open(path, "w") as fd:
        fd.write(data)


def read_file(path):
    global _fake_files
    global _fake_paths
    if _fake_files is not None:
        if path in _fake_files:
            return _fake_files[path]
    if _fake_paths is not None:
        if path in _fake_paths:
            path = _fake_paths[path]
    with open(path) as fd:
        data = fd.read()
    return data


def fake_file(path, data=None, newpath=None):
    """
    Set up a fake file to be read with read_file() in testing
    If data is specified, the string passed as data will be returned instead
    if newpath is specified, the file attempted to be read will be replaced
    by newfile
    """
    global _fake_files
    global _fake_paths
    if data is not None:
        if _fake_files is None:
            _fake_files = {}
        _fake_files[path] = data
    if newpath is not None:
        if _fake_paths is None:
            _fake_paths = {}
        _fake_paths[path] = newpath


def fake_machine(type):
    """
    Set up a fake machine type for testing
    """
    global _fake_machine
    _fake_machine = type


def clear_fakes():
    global _fake_files
    global _fake_paths
    _fake_files = {}
    _fake_paths = {}


def clear_fake_machine():
    global _fake_machine
    _fake_machine = None


def run_and_log(cmd, fd, quiet=False):
    """
    Run a command and log the output to fd
    """
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT, shell=True)
    while proc.returncode == None:
        proc.poll()
        data = proc.stdout.readline()
        fd.write(data)
        if quiet is False:
            sys.stdout.write(data)
    return proc.returncode


def get_machine_type():
    """
    Return the machine type
    """
    global _fake_machine
    if _fake_machine is None:
        return os.uname()[-1]
    return _fake_machine



# Copyright (c) 2010 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abrek.command


def main(argv):
    argv = argv[1:]
    if not argv:
        argv = ['help']
    cmd = argv.pop(0)
    cmd_func = abrek.command.get_command(cmd)
    if not cmd_func:
        print "command '%s' not found" % cmd
        return 1
    return cmd_func.main(argv)

if __name__ == '__main__':
    import os
    import sys
    exit_code = main(sys.argv)
    sys.exit(exit_code)


# Copyright (c) 2010 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abrek.testdef

URL="http://www.cs.virginia.edu/stream/FTP/Code/stream.c"
INSTALLSTEPS = ['cc stream.c -O2 -fopenmp -o stream']
DEPS = ['gcc', 'build-essential']
RUNSTEPS = ['./stream']
PATTERN = "^(?P<test_case_id>\w+):\W+(?P<measurement>\d+\.\d+)"

streaminst = abrek.testdef.AbrekTestInstaller(INSTALLSTEPS, deps=DEPS, url=URL)
streamrun = abrek.testdef.AbrekTestRunner(RUNSTEPS)
streamparser = abrek.testdef.AbrekTestParser(PATTERN,
               appendall={'units':'MB/s', 'result':'pass'})
testobj = abrek.testdef.AbrekTest(testname="stream", installer=streaminst,
                                  runner=streamrun, parser=streamparser)



"""
Public API for extending Abrek
"""
from abc import abstractmethod, abstractproperty

class ITestProvider(object):
    """
    Abrek test provider.

    Abstract source of abrek tests.
    """

    @abstractmethod
    def __init__(self, config):
        """
        Initialize test provider with the specified configuration object. The
        configuration object is obtained from the abrek providers registry.
        """

    @abstractmethod
    def __iter__(self):
        """
        Iterates over instances of ITest exposed by this provider
        """

    @abstractmethod
    def __getitem__(self, test_name):
        """
        Return an instance of ITest with the specified name
        """

    @abstractproperty
    def description(self):
        """
        The description string used by abrek list-tests
        """


class ITest(object):
    """
    Abrek test.

    Something that can be installed and invoked by abre.
    """

    @abstractmethod
    def install(self):
        """
        Install the test suite.

        This creates an install directory under the user's XDG_DATA_HOME
        directory to mark that the test is installed.  The installer's
        install() method is then called from this directory to complete any
        test specific install that may be needed.
        """

    @abstractmethod
    def uninstall(self):
        """
        Uninstall the test suite.

        Uninstalling just recursively removes the test specific directory under
        the user's XDG_DATA_HOME directory.  This will both mark the test as
        removed, and clean up any files that were downloaded or installed under
        that directory.  Dependencies are intentionally not removed by this.
        """

    @abstractmethod
    def run(self, quiet=False):
        # TODO: Document me
        pass
    
    @abstractmethod
    def parse(self, resultname):
        # TODO: Document me
        pass



#!/usr/bin/env python

# Copyright (c) 2010, 2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup, find_packages
from abrek import __version__ as version


setup(
    name='lava-test',
    version=version,
    author='Linaro Validation Team',
    author_email='linaro-dev@lists.linaro.org',
    url='https://launchpad.net/lava-test',
    description='Lava test execution framework',
    long_description=open("README").read(),
    packages=find_packages(exclude=['tests']),
    license="GNU GPLv3",
    test_suite='tests.test_suite',
    scripts = ['bin/lava-test'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.6",
        "Topic :: Software Development :: Testing",
    ],
    zip_safe=False,
    include_package_data=True)




# Copyright (c) 2010 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
from optparse import make_option

import abrek.command
import abrek.testdef
from abrek.config import get_config


class cmd_version(abrek.command.AbrekCmd):
    """
    Show the version of abrek
    """
    def run(self):
        import abrek
        print abrek.__version__


class cmd_help(abrek.command.AbrekCmd):
    """ Get help on abrek commands

    If the command name is ommited, calling the help command will return a
    list of valid commands.
    """
    arglist = ['command', 'subcommand']
    def run(self):
        if len(self.args) < 1:
            print "Available commands:"
            for cmd in abrek.command.get_all_cmds():
                print "  %s" % cmd
            print
            print "To access extended help on a command use 'abrek help " \
                  "[command]'"
            return
        command_name = self.args.pop(0)
        cmd = abrek.command.get_command(command_name)
        if not cmd:
            print "No command found for '%s'" % command_name
            return
        while self.args:
            subcommand_name = self.args.pop(0)
            cmd = cmd.get_subcommand(subcommand_name)
            if not cmd:
                print "No sub-command of '%s' found for '%s'" % (
                    command_name, subcommand_name)
                return
            command_name += ' ' + subcommand_name
        print cmd.help()


class cmd_install(abrek.command.AbrekCmd):
    """
    Install a test
    """
    arglist = ['*testname']

    def run(self):
        self.checkroot()
        if len(self.args) != 1:
            print "please specify the name of the test to install"
            sys.exit(1)
        test = abrek.testdef.testloader(self.args[0])
        try:
            test.install()
        except RuntimeError as strerror:
            print "Test installation error: %s" % strerror
            sys.exit(1)


class cmd_run(abrek.command.AbrekCmd):
    """
    Run tests
    """
    arglist = ['*testname']
    options = [make_option('-q', '--quiet', action='store_true',
                           default=False, dest='quiet'),
               make_option('-o', '--output',  action='store',
                           default=None, metavar="FILE",
                           help="Store processed test output to FILE")]

    def run(self):
        self.checkroot()
        if len(self.args) != 1:
            print "please specify the name of the test to run"
            sys.exit(1)
        test = abrek.testdef.testloader(self.args[0])
        try:
            result_id = test.run(quiet=self.opts.quiet)
            if self.opts.output:
                from abrek.dashboard import generate_bundle
                import json
                bundle = generate_bundle(result_id)
                with open(self.opts.output, "wt") as stream:
                    json.dump(bundle, stream)
        except Exception as strerror:
            print "Test execution error: %s" % strerror
            sys.exit(1)


class cmd_uninstall(abrek.command.AbrekCmd):
    """
    Uninstall a test
    """
    arglist = ['*testname']

    def run(self):
        if len(self.args) != 1:
            print "please specify the name of the test to uninstall"
            sys.exit(1)
        test = abrek.testdef.testloader(self.args[0])
        try:
            test.uninstall()
        except Exception as strerror:
            print "Test uninstall error: %s" % strerror
            sys.exit(1)


class cmd_list_installed(abrek.command.AbrekCmd):
    """
    List tests that are currently installed
    """
    def run(self):
        config = get_config()
        print "Installed tests:"
        try:
            for dir in os.listdir(config.installdir):
                print dir
        except OSError:
            print "No tests installed"


class cmd_list_tests(abrek.command.AbrekCmd):
    """
    List all known tests
    """
    def run(self):
        from abrek.testdef import TestLoader
        for provider in TestLoader().get_providers():
            print provider.description
            for test in provider:
                print " - %s" % test


class cmd_register_test(abrek.command.AbrekCmd):
    """
    Register declarative tests
    """

    arglist = ['test_url']

    def run(self):
        if len(self.args) != 1:
            self.parser.error("You need to provide an URL to a test definition file")
        test_url = self.args[0]
        from abrek.providers import RegistryProvider
        try:
            RegistryProvider.register_remote_test(test_url)
        except ValueError as exc:
            print "Unable to register test: %s" % exc
            sys.exit(1)



# Copyright (c) 2010 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from optparse import OptionParser
import os
import sys


class _AbrekOptionParser(OptionParser):
    """
    This is just to override the epilog formatter to allow newlines
    """
    def format_epilog(self, formatter):
        return self.epilog


class AbrekCmd(object):
    """ Base class for commands that can be passed to Abrek.

    Commands added to abrek should inherit from AbrekCmd.  To allow for
    autodiscovery, the name of the class should begin with cmd_.

    Arguments allowed by the command can be specified in the 'arglist'.
    These arguments will automatically be listed in the help for that
    command.  Required arguments should begin with a '*'.  For example:
        arglist = ['*requiredarg', 'optionalarg']

    Options may also be specified by using the 'options' list.  To add
    arguments, you must use the make_option() function from optparse.
    For example:
        options = [make_option("-b", "--bar", dest="bar")]

    Commands also support subcommands.  A subcommand is similar to a
    command in abrek, and it should also inherit from AbrekCmd.  However,
    a subcommand class should not begin with cmd_.  Instead, it should
    be tied to the command that uses it, using the 'subcmds' dict.
    For example:
        class subcmd_bar(AbrekCmd):
            pass
        class cmd_foo(AbrekCmd):
            subcmds = {'bar':subcmd_bar()}
            pass
    """
    options = []
    arglist = []

    def __init__(self, name_prefix=''):
        self._name_prefix = name_prefix
        self.parser = _AbrekOptionParser(usage=self._usage(),
                                         epilog=self._desc())
        for opt in self.options:
            self.parser.add_option(opt)

    def main(self, argv):
        (self.opts, self.args) = self.parser.parse_args(argv)
        return self.run()

    def name(self):
        return self._name_prefix + _convert_command_name(self.__class__.__name__)

    def run(self):
        raise NotImplementedError("%s: command defined but not implemented!" %
                                  self.name())

    def _usage(self):
        usagestr = "Usage: lava-test %s" % self.name()
        for arg in self.arglist:
            if arg[0] == '*':
                usagestr += " %s" % arg[1:].upper()
            else:
                usagestr += " [%s]" % arg.upper()
        return usagestr

    def _desc(self):
        from inspect import getdoc
        docstr = getdoc(self)
        if not docstr:
            return ""
        description = "\nDescription:\n"
        description += docstr + "\n"
        return description

    def help(self):
        #For some reason, format_help includes an extra \n
        return self.parser.format_help()[:-1]

    def get_subcommand(self, name):
        return None

    def checkroot(self):
        if os.getuid() != 0:
            print >> sys.stderr, ("**** WARNING: ROOT PERMISSIONS ARE OFTEN"
                "REQUIRED FOR THIS OPERATION ****")


class AbrekCmdWithSubcommands(AbrekCmd):

    arglist = ['subcommand']

    def main(self, argv):
        if not argv:
            print "Missing sub-command." + self._list_subcmds()
        else:
            subcmd = self.get_subcommand(argv[0])
            if subcmd is None:
                # This line might print the help and raise SystemExit if
                # --help is passed or if an invalid option was passed.
                opts, args = self.parser.parse_args(argv)
                # If it didn't, complain.
                print "'%s' not found as a sub-command of '%s'" % (
                    args[0], self.name()) + self._list_subcmds()
            else:
                return subcmd.main(argv[1:])

    def get_subcommand(self, name):
        subcmd_cls = getattr(self, 'cmd_' + name.replace('_', '-'), None)
        if subcmd_cls is None:
            return None
        return subcmd_cls(self.name() + ' ')

    def _usage(self):
        usagestr = AbrekCmd._usage(self)
        usagestr += self._list_subcmds()
        return usagestr

    def _list_subcmds(self):
        subcmds = []
        for attrname in self.__class__.__dict__.keys():
            if attrname.startswith('cmd_'):
                subcmds.append(_convert_command_name(attrname))
        if not subcmds:
            return ''
        return "\n\nAvailable sub-commands:\n  " + "\n  ".join(subcmds)


def _convert_command_name(cmd):
    return cmd[4:].replace('_','-')


def _find_commands(module):
    cmds = {}
    for name, func in module.__dict__.iteritems():
        if name.startswith("cmd_"):
            real_name = _convert_command_name(name)
            cmds[real_name] = func()
    return cmds


def get_all_cmds():
    from abrek import builtins, dashboard, results
    cmds = _find_commands(builtins)
    cmds.update(_find_commands(dashboard))
    cmds.update(_find_commands(results))
    return cmds


def get_command(cmd_name):
    cmds = get_all_cmds()
    return cmds.get(cmd_name)



# Copyright (c) 2010,2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abrek.testdef

INSTALLSTEPS = ['git clone git://github.com/janimo/firefox-startup-timing.git']
DEPS = ['firefox', 'git-core', 'gcalctool']
RUNSTEPS = ['cd firefox-startup-timing; ./firefox_startup_timing.sh']
PATTERN = "^(?P<test_case_id>\w+):(?P<measurement>\d+)"

firefoxinst = abrek.testdef.AbrekTestInstaller(INSTALLSTEPS, deps=DEPS)
firefoxrun = abrek.testdef.AbrekTestRunner(RUNSTEPS)
firefoxparser = abrek.testdef.AbrekTestParser(PATTERN,
               appendall={'units':'ms', 'result':'pass'})
testobj = abrek.testdef.AbrekTest(testname="firefox", installer=firefoxinst,
                                  runner=firefoxrun, parser=firefoxparser)



# Copyright (c) 2010 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import base64
import os
import socket
import sys
import urllib
import xmlrpclib
from ConfigParser import ConfigParser, NoOptionError
from getpass import getpass
from optparse import make_option

from abrek.bundle import DocumentIO
from abrek.command import AbrekCmd, AbrekCmdWithSubcommands
from abrek.config import get_config
from abrek.testdef import testloader


class DashboardConfig(object):
    """
    Read and write dashboard configuration data
    """

    def __init__(self, section="Default Server"):
        self.dashboardconf = ConfigParser()
        self.section = section
        self.config = get_config()
        self.path = os.path.join(self.config.configdir, "dashboard.conf")
        if os.path.exists(self.path):
            self.dashboardconf.read(self.path)
        if not (self.section in self.dashboardconf.sections()):
            self.dashboardconf.add_section(self.section)

    def set_host(self, host):
        self.dashboardconf.set(self.section, 'host', host)

    def get_host(self):
        try:
            host = self.dashboardconf.get(self.section, 'host')
            return host
        except NoOptionError:
            return ""

    host = property(get_host, set_host)

    def set_user(self, user):
        self.dashboardconf.set(self.section, 'user', user)

    def get_user(self):
        try:
            user = self.dashboardconf.get(self.section, 'user')
            return user
        except NoOptionError:
            return ""

    user = property(get_user, set_user)

    def set_password(self, password):
        #Not exactly secure, but better than storing in plaintext
        password = base64.encodestring(password).strip()
        self.dashboardconf.set(self.section,'password', password)

    def get_password(self):
        try:
            password = self.dashboardconf.get(self.section, 'password')
            return base64.decodestring(password)
        except NoOptionError:
            return ""

    password = property(get_password, set_password)

    def write(self):
        """
        write the dashboard configuration out to the config file
        """
        if not os.path.exists(self.config.configdir):
            os.makedirs(self.config.configdir)
        with open(self.path, "w") as fd:
            self.dashboardconf.write(fd)


class cmd_dashboard(AbrekCmdWithSubcommands):
    """
    Connect to the Launch-control dashboard
    """

    class cmd_setup(AbrekCmd):
        """
        Configure information needed to push results to the dashboard
        """
        options = [make_option("-u", "--user", dest="user"),
                   make_option("-p", "--password", dest="password")]
        arglist = ["*server"]

        def run(self):
            if len(self.args) != 1:
                print "You must specify a server"
                sys.exit(1)
            config = DashboardConfig()
            if self.opts.user:
                user = self.opts.user
            else:
                user = raw_input("Username: ")
            if self.opts.password:
                password = self.opts.password
            else:
                password = getpass()
            config.host = self.args[0]
            config.user = user
            config.password = password
            config.write()


    class cmd_put(AbrekCmd):
        """
        Push the results from a test to the server
        The stream name must include slashes (e.g. /anonymous/foo/)
        """
        arglist = ["*stream", "*result"]

        def run(self):
            if len(self.args) != 2:
                print "You must specify a stream and a result"
                sys.exit(1)
            stream_name = self.args[0]
            result_name = self.args[1]
            bundle = generate_bundle(result_name)
            db_config = DashboardConfig()
            hosturl = urllib.basejoin(db_config.host, "xml-rpc/")
            try:
                server = xmlrpclib.Server(hosturl)
            except IOError, e:
                print ("Error connecting to server at '%s'. Error was: %s, "
                    "please run 'lava-test dashboard setup [host]'" % (
                        hosturl, e))
                sys.exit(1)
            try:
                result = server.put(DocumentIO.dumps(bundle), result_name,
                    stream_name)
                print "Bundle successfully uploaded to id: %s" % result
            except xmlrpclib.Fault as strerror:
                print "Error uploading bundle: %s" % strerror.faultString
                sys.exit(1)
            except socket.error as strerror:
                print "Unable to connect to host: %s" % strerror
                sys.exit(1)
            except xmlrpclib.ProtocolError as strerror:
                print "Connection error: %s" % strerror
                sys.exit(1)


    class cmd_bundle(AbrekCmd):
        """
        Print JSON output that can be imported into the dashboard
        """
        arglist = ["*result"]

        def run(self):
            if len(self.args) != 1:
                print "You must specify a result"
                sys.exit(1)
            bundle = generate_bundle(self.args[0])
            try:
                print DocumentIO.dumps(bundle)
            except IOError:
                pass


def generate_bundle(result):
    config = get_config()
    resultdir = os.path.join(config.resultsdir, result)
    if not os.path.exists(resultdir):
        # FIXME: UI and sys.exit mixed with internal implementation, yuck
        print "Result directory not found"
        sys.exit(1)
    with open(os.path.join(resultdir, "testdata.json")) as stream:
        bundle_text = stream.read()
    with open(os.path.join(resultdir, "testoutput.log")) as stream:
        output_text = stream.read()
    fmt, bundle = DocumentIO.loads(bundle_text)
    test = testloader(bundle['test_runs'][0]['test_id'])
    test.parse(result)
    bundle['test_runs'][0].update(test.parser.results)
    bundle['test_runs'][0]["attachments"] = [
        {
            "pathname": "testoutput.log",
            "mime_type": "text/plain",
            "content":  base64.standard_b64encode(output_text)
        }
    ]
    return bundle 


